#!/usr/bin/env python
# -*- coding: UTF-8 -*-

'''
* VERSION:1.0
* 2019/1/10
* 支持服务器更新

* VERSION:1.1
* 2019/4/25
* 适配新版本服务器，增加ARQ3的更新 
'''

import getopt, sys, os
import urllib2
import socket
import json
import time

logf = open("/root/update.log", "w")
def LOGT(log):
    global logf
    logf.write("%s\n" % log)

def downFile(ip, filename):
    url = "http://%s/%s" % (ip, filename)
    req = urllib2.Request(url)
    res = urllib2.urlopen(req)
    data = res.read()
    with open("/root/%s" % filename , "w") as code:
        code.write(data)    

    return res.getcode()

def getWanIP(ip):
    try:
        s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM)
        s.connect((ip, 80))
        ip = s.getsockname()[0]
    finally:
        s.close()

    return ip

def usage():
    print("update.py version 1.1")
    print('-l, --lanip')
    print("     调度服务器局域网IP")
    print('-w, --wanip')
    print('     调度服务器外网IP')     
    print('-n, --pname')
    print('     指定进程名')         
    print('-d, --dynamic')
    print('     默认为固定服务器，指定该参数时为弹性服务器')             
    print('-r, --run')
    print('     直接运行不更新自己')                 
    print('-h, --help')
    print('     显示此菜单')
    print ''   

def initGlobalZlog(f):
    f.write("[global]\n")
    f.write("strict init = true\n")
    f.write("buffer min = 1024\n")
    f.write("buffer max = 2MB\n")
    f.write("[formats]\n")
    f.write('txt="%d.%ms | %-10c | %-5V | %p:%T | %f:%L | %m%n"\n')
    f.write("[rules]\n")    

def initZlog(pname, zlog):
    with open("/root/zlog.conf", "w") as f:
        initGlobalZlog(f)
        f.write('%s.* "./log/%s.log", 10M*100; txt\n' % (zlog, pname))
        f.close()

    with open("/root/zlog_d.conf", "w") as f:
        initGlobalZlog(f)
        f.write("%s.* > stdout; txt" % zlog)
        f.close()

def initMoniterByP2PM(lan_ip, wan_ip, servertype):
    #解析turnserver.conf
    bw = 655360
    stat_limit = 5
    device_timeout = 60
    recast_timeout = 300
    client_timeout = 60
    port = 18888
    bindport = 19999
    with open("/root/turnserver.conf", "r") as f:
        conf = json.loads(f.readline())
        if conf.has_key("bw"):
            bw = conf["bw"]
        if conf.has_key("stat_limit"):
            stat_limit = conf["stat_limit"]
        if conf.has_key("client_timeout"):
            client_timeout = conf["client_timeout"]
        if conf.has_key("recast_timeout"):
            recast_timeout = conf["recast_timeout"]
        if conf.has_key("port"):
            port = conf["port"]
        if conf.has_key("bindport"):
            bindport = conf["bindport"]
    #获取外网IP
    ip = getWanIP(wan_ip)
    LOGT("IP : %s" % ip)
    #生成moniter.conf
    with open("/root/moniter.conf", "w") as f:
        f.write("[main]\n")
        f.write("fifo=turnserver_fifo\n")
        f.write("run=/root/turnserver -i %s -p %d -n %s -o %d %s -T %d -R %d -w %d\n" % (lan_ip, port, ip, bindport, servertype, client_timeout, recast_timeout, bw))
        f.write("pname=turnserver\n")
        f.close()

def initMoniterByP2PS(lan_ip, wan_ip):
    bindport = 9000
    port = 18888
    netname = "eth0"
    with open("/root/p2pserver.conf","r") as f:
        conf = json.loads(f.readline())
        if conf.has_key("bindport"):
            bindport = conf["bindport"]
        if conf.has_key("port"):
            port = conf["port"]
        if conf.has_key("netname"):
            netname = conf["netname"]

    with open("/root/moniter.conf", "w") as f:
        f.write("[main]\n")
        f.write("fifo=p2pserver_fifo\n")
        f.write("run=/root/p2pserver -m %s -n %d -p %d -a %s\n" % (lan_ip, port, bindport, netname))
        f.write("pname=p2pserver\n")
        f.close()


'''
转发更新脚本，更新转发程序，配置文件，生成moniter配置文件，并启动moniter
'''
if __name__ == "__main__":

    lan_ip = "10.26.173.142"
    wan_ip = "139.129.204.138"   
    pname = "" 
    zlog = ""
    servertype = ""
    brun = False

    try:
        opts, args = getopt.getopt(sys.argv[1:], 'l:w:n:rdh',
                                   ['lanip','wanip','pname','run','dynamic','help'])
    except getopt.GetoptError as err:
        print(str(err))
        usage()
        sys.exit(-1)   

    cmd_args = ""
    for o, a in opts:
        if o in ('-h', '--help'):
            usage()
            sys.exit(0)
        elif o in ('-l', '--lanip'):
            lan_ip = a
            cmd_args = cmd_args + " -l " + a
        elif o in ('-w', '--wanip'):
            wan_ip = a
            cmd_args = cmd_args + " -w " + a
        elif o in ('-n', '--pname'):
            pname = a.strip()
            cmd_args = cmd_args + " -n " + a
        elif o in ('-d', '--dynamic'):
            servertype = "-t"
            cmd_args = cmd_args + " -d"
        elif o in ('-r', '--run'):
            brun = True

    if pname == "":
        usage()
        sys.exit(0)

    if pname == "turnserver":
        zlog = "KP2PMS"
    elif pname == "p2pserver":
        zlog = "P2PS"

    if zlog == "":
        LOGT("invalid pname")
        usage()
        sys.exit(0)

    if not brun:
        code = downFile(lan_ip, "update.py")
        if(code == 200):
            os.system("python /root/update.py -r " + cmd_args)
            sys.exit(0)

    #如果存在，则关闭进程
    if os.path.exists("stop.sh"):
        os.popen("/root/stop.sh %s -a" % pname)
    #下载libarq.so
    timeout = 0
    while(timeout < 5):
        code = downFile(lan_ip, "libarq.so")
        LOGT("libarq.so : %d" % code)
        if code == 200:
            break
        else:
            lan_ip = wan_ip

        timeout += 1
    #安装arq至lib64
    os.system("mv libarq.so /lib64/libarq.so")
    
    #使用内网IP部署转发
    #安装moniter
    timeout = 0
    while(timeout < 5):
        code = downFile(lan_ip, "moniter")
        LOGT("moniter : %d " % code)
        if code == 200:
            break
        else:
            lan_ip = wan_ip
        timeout += 1
    #安装stop.sh
    timeout = 0
    while(timeout < 5):
        code = downFile(lan_ip, "stop.sh")
        LOGT("stop.sh : %d " % code)
        if code == 200:
            break
        timeout += 1
    #安装主进程
    timeout = 0
    while(timeout < 5):
        code = downFile(lan_ip, pname)
        LOGT("%s : %d " % (pname, code))
        if code == 200:
            break
        timeout += 1
    # #安装zlog.conf
    # timeout = 0
    # while(timeout < 5):
    #     code = downFile(lan_ip, "zlog.conf")
    #     print "zlog.conf : %d " % code 
    #     if code == 200:
    #         break
    #     timeout += 1
    # #安装zlog_d.conf
    # timeout = 0
    # while(timeout < 5):
    #     code = downFile(lan_ip, "zlog_d.conf")
    #     print "zlog_d.conf : %d " % code
    #     if code == 200:
    #         break
    #     timeout += 1

    #安装zlog
    initZlog(pname, zlog)

    #安装主进程.conf
    timeout = 0
    while (timeout < 5):
        code = downFile(lan_ip, "%s.conf" % pname)        
        LOGT("%s.conf : %d " % (pname, code))
        if code == 200:
            break
        timeout += 1

    if pname == "turnserver":
        initMoniterByP2PM(lan_ip, wan_ip, servertype)

    elif pname == "p2pserver":
        initMoniterByP2PS(lan_ip, wan_ip)

    #创建log目录
    if not os.path.exists("/root/log"):
        os.mkdir("/root/log")
    #启动moniter
    os.system("chmod 777 /root/stop.sh")
    os.system("chmod 777 /root/%s" % pname)
    os.system("chmod 777 /root/moniter")
    os.chdir("/root")
    os.popen("/root/moniter")
    LOGT("run /root/moniter")
    logf.close()
